from ...resources import Resources
from hardeneks.rules import Rule, Result


class use_encryption_with_ebs(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "encryption_secrets"
    message = "EBS Storage Classes should have encryption parameter."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/data/#encryption-at-rest"

    def check(self, resources: Resources):
        offenders = []

        for storage_class in resources.storage_classes:
            if storage_class.provisioner == "ebs.csi.aws.com":
                encrypted = storage_class.parameters.get("encrypted")
                if not encrypted:
                    offenders.append(storage_class)
                elif encrypted == "false":
                    offenders.append(storage_class)

        self.result = Result(status=True, resource_type="StorageClass")

        if offenders:
            self.result = Result(
                status=False,
                resource_type="StorageClass",
                resources=[i.metadata.name for i in offenders],
            )


class use_encryption_with_efs(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "encryption_secrets"
    message = "EFS Persistent volumes should have tls mount option."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/data/#encryption-at-rest"

    def check(self, resources: Resources):

        offenders = []

        for persistent_volume in resources.persistent_volumes:
            csi = persistent_volume.spec.csi
            if csi and csi.driver == "efs.csi.aws.com":
                mount_options = persistent_volume.spec.mount_options
                if not mount_options:
                    offenders.append(persistent_volume)
                else:
                    if "tls" not in mount_options:
                        offenders.append(persistent_volume)

        self.result = Result(status=True, resource_type="PersistentVolume")

        if offenders:
            self.result = Result(
                status=False,
                resource_type="PersistentVolume",
                resources=[i.metadata.name for i in offenders],
            )


class use_efs_access_points(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "encryption_secrets"
    message = "EFS Persistent volumes should leverage access points."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/data/#use-efs-access-points-to-simplify-access-to-shared-datasets"

    def check(self, resources: Resources):

        offenders = []

        for persistent_volume in resources.persistent_volumes:
            csi = persistent_volume.spec.csi
            if csi and csi.driver == "efs.csi.aws.com":
                if "::" not in csi.volume_handle:
                    offenders.append(persistent_volume)

        self.result = Result(status=True, resource_type="PersistentVolume")

        if offenders:
            self.result = Result(
                status=False,
                resource_type="PersistentVolume",
                resources=[i.metadata.name for i in offenders],
            )



from abc import ABC, abstractmethod

from hardeneks import console


class Result(object):
    def __init__(
        self, status=True, resources=[""], resource_type=None, namespace=None
    ):
        self.status = status
        self.resources = resources
        self.resource_type = resource_type
        self.namespace = namespace


class Rule(ABC):

    message = None
    url = None
    _type = None
    pillar = None
    section = None
    console = console

    def __init__(self, result=Result()):
        self.result = result

        if not (hasattr(self, "message") and self.message):
            raise NotImplementedError(
                "Class needs to have class variable message"
            )
        if not (hasattr(self, "url") and self.url):
            raise NotImplementedError("Class needs to have class variable url")
        if not (hasattr(self, "_type") and self._type):
            raise NotImplementedError(
                "Class needs to have class variable _type"
            )
        if not (hasattr(self, "pillar") and self.pillar):
            raise NotImplementedError(
                "Class needs to have class variable pillar"
            )
        if not (hasattr(self, "section") and self.section):
            raise NotImplementedError(
                "Class needs to have class variable section"
            )

    @abstractmethod
    def check(self):
        pass



from ...resources import NamespacedResources
from hardeneks.rules import Rule, Result


class disallow_secrets_from_env_vars(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "encryption_secrets"
    message = "Disallow secrets from env vars."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/data/#use-volume-mounts-instead-of-environment-variables"

    def check(self, namespaced_resources: NamespacedResources):
        offenders = []

        for pod in namespaced_resources.pods:
            for container in pod.spec.containers:
                if container.env:
                    for env in container.env:
                        if env.value_from and env.value_from.secret_key_ref:
                            offenders.append(pod)
                if container.env_from:
                    for env_from in container.env_from:
                        if env_from.secret_ref:
                            offenders.append(pod)

        self.result = Result(status=True, resource_type="Pod")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Pod",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )



from collections import Counter

from ...resources import NamespacedResources
from hardeneks.rules import Rule, Result


class restrict_wildcard_for_roles(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "iam"
    message = "Roles should not have '*' in Verbs or Resources."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#employ-least-privileged-access-when-creating-rolebindings-and-clusterrolebindings"

    def check(self, namespaced_resources: NamespacedResources):
        offenders = []

        for role in namespaced_resources.roles:
            for rule in role.rules:
                if "*" in rule.verbs:
                    offenders.append(role)
                if "*" in rule.resources:
                    offenders.append(role)

        self.result = Result(status=True, resource_type="Role")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Role",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class disable_service_account_token_mounts(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "iam"
    message = "Auto-mounting of Service Account tokens is not allowed."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#disable-auto-mounting-of-service-account-tokens"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        for pod in namespaced_resources.pods:
            if pod.spec.automount_service_account_token:
                offenders.append(pod)

        self.result = Result(status=True, resource_type="Pod")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Pod",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class disable_run_as_root_user(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "iam"
    message = "Running as root is not allowed."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#run-the-application-as-a-non-root-user"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        for pod in namespaced_resources.pods:
            security_context = pod.spec.security_context
            if (
                not security_context.run_as_group
                and not security_context.run_as_user
            ):
                offenders.append(pod)

        self.result = Result(status=True, resource_type="Pod")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Pod",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class disable_anonymous_access_for_roles(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "iam"
    message = "Don't bind roles to anonymous or unauthenticated groups."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#review-and-revoke-unnecessary-anonymous-access"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        for role_binding in namespaced_resources.role_bindings:
            if role_binding.subjects:
                for subject in role_binding.subjects:
                    if (
                        subject.name == "system:unauthenticated"
                        or subject.name == "system:anonymous"
                    ):
                        offenders.append(role_binding)

        self.result = Result(status=True, resource_type="RoleBinding")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="RoleBinding",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class use_dedicated_service_accounts_for_each_deployment(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "iam"
    message = "Don't share service accounts between Deployments."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#use-dedicated-service-accounts-for-each-application"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        count = Counter(
            [
                i.spec.template.spec.service_account_name
                for i in namespaced_resources.deployments
            ]
        )
        repeated_service_accounts = {
            x: count for x, count in count.items() if count > 1
        }

        for k, v in repeated_service_accounts.items():
            for deployment in namespaced_resources.deployments:
                if k == deployment.spec.template.spec.service_account_name:
                    offenders.append(deployment)

        self.result = Result(status=True, resource_type="Deployment")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Deployment",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class use_dedicated_service_accounts_for_each_stateful_set(
    Rule,
):
    _type = "namespace_based"
    pillar = "security"
    section = "iam"
    message = "Don't share service accounts between StatefulSets."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#use-dedicated-service-accounts-for-each-application"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        count = Counter(
            [
                i.spec.template.spec.service_account_name
                for i in namespaced_resources.stateful_sets
            ]
        )
        repeated_service_accounts = {
            x: count for x, count in count.items() if count > 1
        }

        for k, v in repeated_service_accounts.items():
            for deployment in namespaced_resources.stateful_sets:
                if k == deployment.spec.template.spec.service_account_name:
                    offenders.append(deployment)

        self.result = Result(status=True, resource_type="StatefulSet")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="StatefulSet",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class use_dedicated_service_accounts_for_each_daemon_set(
    Rule,
):
    _type = "namespace_based"
    pillar = "security"
    section = "iam"
    message = "Don't share service accounts between DaemonSets."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#use-dedicated-service-accounts-for-each-application"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        count = Counter(
            [
                i.spec.template.spec.service_account_name
                for i in namespaced_resources.daemon_sets
            ]
        )
        repeated_service_accounts = {
            x: count for x, count in count.items() if count > 1
        }

        for k, v in repeated_service_accounts.items():
            for deployment in namespaced_resources.daemon_sets:
                if k == deployment.spec.template.spec.service_account_name:
                    offenders.append(deployment)

        self.result = Result(status=True, resource_type="DaemonSet")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="DaemonSet",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )



from ...resources import Resources
from hardeneks.rules import Rule, Result


class ensure_namespace_quotas_exist(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "multi_tenancy"
    message = "Namespaces should have quotas assigned."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/multitenancy/#namespaces"

    def check(self, resources: Resources):
        offenders = resources.namespaces

        for quota in resources.resource_quotas:
            offenders.remove(quota.metadata.namespace)

        self.result = Result(status=True, resource_type="Namespace")
        if offenders:
            self.result = Result(
                status=False, resources=offenders, resource_type="Namepsace"
            )



import boto3
from kubernetes import client

from hardeneks.rules import Rule, Result
from ...resources import Resources


class restrict_wildcard_for_cluster_roles(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "iam"
    message = "ClusterRoles should not have '*' in Verbs or Resources."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#employ-least-privileged-access-when-creating-rolebindings-and-clusterrolebindings"

    def check(self, resources: Resources):

        offenders = []
        self.result = Result(status=True, resource_type="Cluster Role")

        allow_list = [
            "aws-node",
            "cluster-admin",
            "eks:addon-manager",
            "eks:cloud-controller-manager",
        ]

        for role in resources.cluster_roles:
            role_name = role.metadata.name
            if not (role_name.startswith("system") or role_name in allow_list):
                for rule in role.rules:
                    if "*" in rule.verbs:
                        offenders.append(role_name)
                    if rule.resources and "*" in rule.resources:
                        offenders.append(role_name)

        if offenders:
            self.result = Result(
                status=False,
                resources=offenders,
                resource_type="Cluster Role",
            )


class check_endpoint_public_access(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "iam"
    message = "EKS Cluster Endpoint is not Private."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#make-the-eks-cluster-endpoint-private"

    def check(self, resources: Resources):
        client = boto3.client("eks", region_name=resources.region)
        cluster_metadata = client.describe_cluster(name=resources.cluster)
        endpoint_access = cluster_metadata["cluster"]["resourcesVpcConfig"][
            "endpointPublicAccess"
        ]
        self.result = Result(status=True, resource_type="Cluster Endpoint")

        if endpoint_access:
            self.result = Result(
                status=False, resource_type="Cluster Endpoint"
            )


class check_aws_node_daemonset_service_account(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "iam"
    message = "Update the aws-node daemonset to use IRSA."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#update-the-aws-node-daemonset-to-use-irsa"

    def check(self, resources: Resources):
        daemonset = client.AppsV1Api().read_namespaced_daemon_set(
            name="aws-node", namespace="kube-system"
        )
        self.result = Result(status=True, resource_type="Daemonset")
        if daemonset.spec.template.spec.service_account_name == "aws-node":
            self.result = Result(
                status=False, resources=["aws-node"], resource_type="Daemonset"
            )


class check_access_to_instance_profile(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "iam"
    message = "Restrict access to the instance profile assigned to nodes."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#when-your-application-needs-access-to-imds-use-imdsv2-and-increase-the-hop-limit-on-ec2-instances-to-2"

    def check(self, resources: Resources):
        client = boto3.client("ec2", region_name=resources.region)
        offenders = []

        instance_metadata = client.describe_instances(
            Filters=[
                {
                    "Name": "tag:aws:eks:cluster-name",
                    "Values": [
                        resources.cluster,
                    ],
                },
            ]
        )

        for instance in instance_metadata["Reservations"]:
            if (
                instance["Instances"][0]["MetadataOptions"][
                    "HttpPutResponseHopLimit"
                ]
                == 2
            ):
                offenders.append(instance)

        self.result = Result(status=True, resource_type="Node")

        if offenders:
            self.result = Result(
                status=False,
                resource_type="Node",
                resources=[i["Instances"][0]["InstanceId"] for i in offenders],
            )


class disable_anonymous_access_for_cluster_roles(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "iam"
    message = "Don't bind clusterroles to anonymous/unauthenticated groups."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/iam/#review-and-revoke-unnecessary-anonymous-access"

    def check(self, resources: Resources):
        offenders = []

        ignored = ["system:public-info-viewer"]

        for cluster_role_binding in resources.cluster_role_bindings:
            if (
                cluster_role_binding.subjects
                and cluster_role_binding.metadata.name not in ignored
            ):
                for subject in cluster_role_binding.subjects:
                    if (
                        subject.name == "system:unauthenticated"
                        or subject.name == "system:anonymous"
                    ):
                        offenders.append(cluster_role_binding)

        self.result = Result(status=True, resource_type="ClusterRoleBinding")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="ClusterRoleBinding",
                resources=[i.metadata.name for i in offenders],
            )



import boto3
from kubernetes import client

from hardeneks.rules import Rule, Result
from ...resources import Resources


def _get_policy_documents_for_role(role_name, iam_client):
    attached_policies = iam_client.list_attached_role_policies(
        RoleName=role_name
    )["AttachedPolicies"]
    inline_policies = iam_client.list_role_policies(RoleName=role_name)[
        "PolicyNames"
    ]
    actions = []
    for policy_arn in [x["PolicyArn"] for x in attached_policies]:
        version_id = iam_client.get_policy(PolicyArn=policy_arn)["Policy"][
            "DefaultVersionId"
        ]
        response = iam_client.get_policy_version(
            PolicyArn=policy_arn, VersionId=version_id
        )["PolicyVersion"]["Document"]["Statement"]
        for statement in response:
            if type(statement["Action"]) == str:
                actions.append(statement["Action"])
            elif type(statement["Action"]) == list:
                actions.extend(statement["Action"])
    for policy_name in inline_policies:
        response = iam_client.get_role_policy(
            RoleName=role_name, PolicyName=policy_name
        )["PolicyDocument"]["Statement"]
        for statement in response:
            if type(statement["Action"]) == str:
                actions.append(statement["Action"])
            elif type(statement["Action"]) == list:
                actions.extend(statement["Action"])
    return actions


class check_any_cluster_autoscaler_exists(Rule):
    _type = "cluster_wide"
    pillar = "cluster_autoscaling"
    section = "cluster_autoscaler"
    message = "Cluster Autoscaler or Karpenter is not deployed."
    url = "https://aws.github.io/aws-eks-best-practices/cluster-autoscaling/"

    def check(self, resources: Resources):
        deployments = [
            i.metadata.name
            for i in client.AppsV1Api()
            .list_deployment_for_all_namespaces()
            .items
        ]
        if not (
            "cluster-autoscaler" in deployments or "karpenter" in deployments
        ):
            self.result = Result(status=False, resource_type="Deployment")
        else:
            self.result = Result(status=True, resource_type="Deployment")

        return self.result


class ensure_cluster_autoscaler_and_cluster_versions_match(Rule):
    _type = "cluster_wide"
    pillar = "cluster_autoscaling"
    section = "cluster_autoscaler"
    message = (
        "Cross version compatibility between CA and k8s is not recommended."
    )
    url = "https://aws.github.io/aws-eks-best-practices/cluster-autoscaling/#operating-the-cluster-autoscaler"

    def check(self, resources):
        eks_client = boto3.client("eks", region_name=resources.region)
        cluster_metadata = eks_client.describe_cluster(name=resources.cluster)

        cluster_version = cluster_metadata["cluster"]["version"]

        deployments = (
            client.AppsV1Api().list_deployment_for_all_namespaces().items
        )

        self.result = Result(status=True, resource_type="Deployment")

        for deployment in deployments:
            if deployment.metadata.name == "cluster-autoscaler":
                ca_containers = deployment.spec.template.spec.containers
                ca_image = ca_containers[0].image
                ca_image_version = ca_image.split(":")[-1]
                if cluster_version not in ca_image_version:
                    self.result = Result(
                        status=False, resource_type="Deployment"
                    )


class ensure_cluster_autoscaler_has_autodiscovery_mode(Rule):
    _type = "cluster_wide"
    pillar = "cluster_autoscaling"
    section = "cluster_autoscaler"
    message = "Auto discovery is not enabled for Cluster Autoscaler."
    url = "https://aws.github.io/aws-eks-best-practices/cluster-autoscaling/#operating-the-cluster-autoscaler"

    def check(self, resources):
        deployments = (
            client.AppsV1Api().list_deployment_for_all_namespaces().items
        )

        self.result = Result(status=True, resource_type="Deployment")

        for deployment in deployments:
            if deployment.metadata.name == "cluster-autoscaler":
                ca_containers = deployment.spec.template.spec.containers
                ca_command = ca_containers[0].command
                if not any(
                    "node-group-auto-discover" in item for item in ca_command
                ):
                    self.result = Result(
                        status=False, resource_type="Deployment"
                    )
                else:
                    break


class use_separate_iam_role_for_cluster_autoscaler(Rule):
    _type = "cluster_wide"
    pillar = "cluster_autoscaling"
    section = "cluster_autoscaler"
    message = "Cluster-autoscaler deployment does not use a dedicated IAM Role (IRSA)."
    url = "https://aws.github.io/aws-eks-best-practices/cluster-autoscaling/#employ-least-privileged-access-to-the-iam-role"

    def check(self, resources):
        deployments = (
            client.AppsV1Api().list_deployment_for_all_namespaces().items
        )

        self.result = Result(status=True, resource_type="Deployment")

        for deployment in deployments:
            if deployment.metadata.name == "cluster-autoscaler":
                service_account = (
                    deployment.spec.template.spec.service_account_name
                )
                sa_data = client.CoreV1Api().read_namespaced_service_account(
                    service_account, "kube-system", pretty="true"
                )
                if (
                    "eks.amazonaws.com/role-arn"
                    not in sa_data.metadata.annotations.keys()
                ):
                    self.result = Result(
                        status=False, resource_type="Deployment"
                    )
                else:
                    break


class employ_least_privileged_access_cluster_autoscaler_role(Rule):
    _type = "cluster_wide"
    pillar = "cluster_autoscaling"
    section = "cluster_autoscaler"
    message = "Cluster autoscaler role has unnecessary actions assigned."
    url = "https://aws.github.io/aws-eks-best-practices/cluster-autoscaling/#employ-least-privileged-access-to-the-iam-role"

    def check(self, resources):

        deployments = (
            client.AppsV1Api().list_deployment_for_all_namespaces().items
        )

        iam_client = boto3.client("iam", region_name=resources.region)

        ACTIONS = {
            "autoscaling:DescribeAutoScalingGroups",
            "autoscaling:DescribeAutoScalingInstances",
            "autoscaling:DescribeLaunchConfigurations",
            "autoscaling:DescribeScalingActivities",
            "autoscaling:DescribeTags",
            "ec2:DescribeImages",
            "ec2:DescribeInstanceTypes",
            "ec2:DescribeLaunchTemplateVersions",
            "ec2:GetInstanceTypesFromInstanceRequirements",
            "eks:DescribeNodegroup",
            "autoscaling:SetDesiredCapacity",
            "autoscaling:TerminateInstanceInAutoScalingGroup",
        }
        self.result = Result(status=True, resource_type="IAM Role Action")

        for deployment in deployments:
            if deployment.metadata.name == "cluster-autoscaler":
                service_account = (
                    deployment.spec.template.spec.service_account_name
                )
                sa_data = client.CoreV1Api().read_namespaced_service_account(
                    service_account, "kube-system", pretty="true"
                )
                if (
                    "eks.amazonaws.com/role-arn"
                    not in sa_data.metadata.annotations.keys()
                ):
                    break
                else:

                    sa_iam_role_arn = sa_data.metadata.annotations[
                        "eks.amazonaws.com/role-arn"
                    ]
                    sa_iam_role = sa_iam_role_arn.split("/")[-1]
                    actions = _get_policy_documents_for_role(
                        sa_iam_role, iam_client
                    )

                    if len(set(actions) - ACTIONS) > 0:
                        self.result = Result(
                            status=False,
                            resource_type="IAM Role Action",
                            resources=(set(actions) - ACTIONS),
                        )
                    else:
                        self.result = Result(
                            status=True, resource_type="IAM Role Action"
                        )


class use_managed_nodegroups(Rule):
    _type = "cluster_wide"
    pillar = "cluster_autoscaling"
    section = "cluster_autoscaler"
    message = "Nodes are recommended to be part of a managed noge group."
    url = "https://aws.github.io/aws-eks-best-practices/cluster-autoscaling/#configuring-your-node-groups"

    def check(self, resources):
        offenders = []
        nodes = client.CoreV1Api().list_node().items

        for node in nodes:
            labels = node.metadata.labels
            if "eks.amazonaws.com/nodegroup" in labels.keys():
                pass
            elif "alpha.eksctl.io/nodegroup-name" in labels.keys():
                offenders.append(node)
            elif "karpenter.sh/provisioner-name" in labels.keys():
                pass
            else:
                offenders.append(node)

        self.result = Result(status=True, resource_type="Node")

        if offenders:
            self.result = Result(
                status=False,
                resource_type="Node",
                resources=[i.metadata.name for i in offenders],
            )



from pathlib import Path
import urllib3
import yaml

#
# get_kube_config
# returns kube config in json
# 
# we need to update this function to take in a config string, so users can pass in kubeconfig as a param
def get_kube_config():
    urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
    # need to fix this, so user can pass in .kube/config as a param (joshkurz)
    kube_config_orig = f"{Path.home()}/.kube/config"

    with open(kube_config_orig, "r") as fd:
        kubeconfig = yaml.safe_load(fd)
    return kubeconfig


from hardeneks.rules import Rule, Result
from ...resources import NamespacedResources


class disallow_container_socket_mount(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "pod_security"
    message = "Container socket mounts are not allowed."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/pods/#never-run-docker-in-docker-or-mount-the-socket-in-the-container"

    def check(self, namespaced_resources: NamespacedResources):
        offenders = []

        sockets = [
            "/var/run/docker.sock",
            "/var/run/containerd.sock",
            "/var/run/crio.sock",
        ]

        for pod in namespaced_resources.pods:
            for volume in pod.spec.volumes:
                if volume.host_path and volume.host_path.path in sockets:
                    offenders.append(pod)

        self.result = Result(status=True, resource_type="Pod")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Pod",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class disallow_host_path_or_make_it_read_only(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "pod_security"
    message = "Restrict the use of hostpath."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/pods/#restrict-the-use-of-hostpath-or-if-hostpath-is-necessary-restrict-which-prefixes-can-be-used-and-configure-the-volume-as-read-only"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        for pod in namespaced_resources.pods:
            for volume in pod.spec.volumes:
                if volume.host_path:
                    offenders.append(pod)

        self.result = Result(status=True, resource_type="Pod")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Pod",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class set_requests_limits_for_containers(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "pod_security"
    message = "Set requests and limits for each container."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/pods/#set-requests-and-limits-for-each-container-to-avoid-resource-contention-and-dos-attacks"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        for pod in namespaced_resources.pods:
            for container in pod.spec.containers:
                if not (
                    container.resources.limits and container.resources.requests
                ):
                    offenders.append(pod)

        self.result = Result(status=True, resource_type="Pod")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Pod",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class disallow_privilege_escalation(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "pod_security"
    message = "Set allowPrivilegeEscalation in the pod spec to false."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/pods/#do-not-allow-privileged-escalation"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []

        for pod in namespaced_resources.pods:
            for container in pod.spec.containers:
                if (
                    container.security_context
                    and container.security_context.allow_privilege_escalation
                ):
                    offenders.append(pod)

        self.result = Result(status=True, resource_type="Pod")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Pod",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )


class check_read_only_root_file_system(Rule):
    _type = "namespace_based"
    pillar = "security"
    section = "pod_security"
    message = "Configure your images with a read-only root file system."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/pods/#configure-your-images-with-read-only-root-file-system"

    def check(self, namespaced_resources: NamespacedResources):

        offenders = []
        for pod in namespaced_resources.pods:
            for container in pod.spec.containers:
                if (
                    container.security_context
                    and not container.security_context.read_only_root_filesystem
                ):
                    offenders.append(pod)
        self.result = Result(status=True, resource_type="Pod")
        if offenders:
            self.result = Result(
                status=False,
                resource_type="Pod",
                resources=[i.metadata.name for i in offenders],
                namespace=namespaced_resources.namespace,
            )



import kubernetes

from ...resources import Resources
from hardeneks.rules import Rule, Result


class ensure_namespace_psa_exist(Rule):
    _type = "cluster_wide"
    pillar = "security"
    section = "pod_security"
    message = "Namespaces should have psa modes."
    url = "https://aws.github.io/aws-eks-best-practices/security/docs/pods/#pod-security-standards-pss-and-pod-security-admission-psa"

    def check(self, resources: Resources):
        offenders = []

        namespaces = kubernetes.client.CoreV1Api().list_namespace().items
        psa_labels = [
            "pod-security.kubernetes.io/enforce",
            "pod-security.kubernetes.io/warn",
            "pod-security.kubernetes.io/audit",
        ]

        for namespace in namespaces:
            if namespace.metadata.name in resources.namespaces:
                labels = namespace.metadata.labels.keys()
                if not any(i in labels for i in psa_labels):
                    offenders.append(namespace.metadata.name)

        self.result = Result(status=True, resource_type="Namespace")
        if offenders:
            self.result = Result(
                status=False, resource_type="Namespace", resources=offenders
            )
